package com.brilliance.eibs.server;

import com.brilliance.eibs.bean.DynamicCompiler;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.util.ClassPathUpdater;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.io.IOUtils;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;

import javax.servlet.Servlet;
import javax.tools.DiagnosticCollector;
import javax.tools.JavaFileObject;
import java.io.File;
import java.io.FileOutputStream;
import java.lang.reflect.Constructor;
import java.net.URL;
import java.net.URLDecoder;

/**
 * http服务
 * 
 * @author xiaoyuanzhen
 *
 */
public class HttpServer extends AbsServer {

	private Server server;

	public HttpServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
		super(context, serviceDef, elParser);
	}

	@Override
	public void run() {
		logger.info( LOG_FLAG + "starting HttpService[id:" + serviceDef.getId() + "]." + LOG_FLAG);
		try {
			// String ip=getRequiredPropertyValue(IP_SYMBOL);
			String port = getRequiredPropertyValue(PORT_SYMBOL);
			String uri = getRequiredPropertyValue(URI_SYMBOL);
			String classesname = getRequiredPropertyValue(CLASS_SYMBOL);
			String[] clsarray = classesname.split(",");
			// 初始化参数
			String initparam = getPropertyValue(INITPARA);
			String interfaceNam = getPropertyValue("interfaceName");

			if (serviceDef.getId().startsWith("_")) {
				Server server = new Server(Integer.parseInt(port));
				ServletContextHandler context1 = new ServletContextHandler(ServletContextHandler.SESSIONS);
				context1.setContextPath("/");
				context1.setResourceBase(".");
				context1.setClassLoader(Thread.currentThread().getContextClassLoader());

				Constructor c1 = Class.forName(clsarray[0]).getDeclaredConstructor(String.class, String.class, Object[].class);

				String transactionsNames = getPropertyValue("transactionName");

				Servlet b = (Servlet) c1.newInstance(interfaceNam, transactionsNames, args);

				context1.addServlet(new ServletHolder(b), uri);

				ContextHandlerCollection contexts = new ContextHandlerCollection();
				contexts.setHandlers(new Handler[] { context1 });

				server.setHandler(contexts);
				server.start();
				server.join();

				return;
			}
			if (serviceDef.getId().startsWith("$")) {
				Server server = new Server(Integer.parseInt(port));
				ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
				context.setContextPath("/");
				context.setResourceBase(".");
				context.setClassLoader(Thread.currentThread().getContextClassLoader());
				String interfaceNames[] = getPropertyValue("interfaceName").split(",");
				String transactionsNames[] = getPropertyValue("transactionName").split(",");
				String[] uris = getRequiredPropertyValue(URI_SYMBOL).split(",");
				server.setHandler(context);
				for (int i = 0; i < clsarray.length; i++) {
					Constructor c1 = Class.forName(clsarray[i]).getDeclaredConstructor(String.class, String.class, Object[].class);
					Servlet b = (Servlet) c1.newInstance(interfaceNames[i], transactionsNames[i], args);
					context.addServlet(new ServletHolder(b), uris[i]);
				}
				server.start();
				server.join();
				return;
			}
			// 动态生成
			if (!StringUtil.isEmpty(interfaceNam)) {
				String interfaceNames[] = getPropertyValue("interfaceName").split(",");
				String transactionsNames[] = getPropertyValue("transactionName").split(",");
				String[] uris = getRequiredPropertyValue(URI_SYMBOL).split(",");
				URL url = Thread.currentThread().getContextClassLoader().getResource("classes/BaseServlet.base");
				String basestr = IOUtils.toString(url);
				for (int i = 0; i < interfaceNames.length; i++) {
					String clsnam = clsarray[i];
					String c = clsnam.substring(clsnam.lastIndexOf(".") + 1);
					String s = basestr.replaceAll("BaseServlet", c).replace("base1", interfaceNames[i]).replace("base2", transactionsNames[i])
							.replace("//", "");
					String path = Thread.currentThread().getContextClassLoader().getResource("").toURI().getPath() + "classes/classes/" + c + ".java";
					String spath = Thread.currentThread().getContextClassLoader().getResource("").toURI().getPath() + "classes/classes/";
					File f = new File(spath);
					if (!f.exists())
						f.mkdirs();
					FileOutputStream fos = new FileOutputStream(path);
					fos.write(s.getBytes());
					fos.close();
				}
				generateBeans();

				server = new Server(Integer.parseInt(port));
				ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
				context.setContextPath("/");
				server.setHandler(context);

				for (int i = 0; i < clsarray.length; i++) {

					String clsnam = clsarray[i];
					// Class cls = Class.forName(clsnam);
					Class cls = ClassPathUpdater.dynamicLoadClass(clsnam);
					// Object object = cls.newInstance();
					context.addServlet(new ServletHolder(cls), uris[i]);
					// context.addServlet(new ServletHolder, pathSpec)
				}
				server.start();
				server.join();
			}
			// 静态调用API启动Servlet服务.
			else {
				server = new Server(Integer.parseInt(port));
				ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
				context.setContextPath("/");
				server.setHandler(context);
				String initarray[] = null;

				if (!StringUtil.isEmpty(initparam))
					initarray = initparam.split("#");

				for (int i = 0; i < clsarray.length; i++) {
					String clsnam = clsarray[i];
					String param = initarray[i];
					Class cls = Class.forName(clsnam);
					ServletHolder holder = new ServletHolder(cls);
					if (!StringUtil.isEmpty(param)) {
						String m[] = param.split(",");
						for (int j = 0; j < m.length; j++) {
							String s[] = m[j].split("\\|");
							holder.setInitParameter(s[0], s[1]);
						}
						// logger.info(this.context, context,
						// "init Parameters of[" + this.getClass().getName() +
						// "] is:" + holder.getInitParameters());

					}
					context.addServlet(holder, uri);
				}
				server.start();
				server.join();
			}

		} catch (Exception e) {
			logger.error( "jetty服务初始化异常:"+e.getMessage(),e);
			throw new InterfaceException("01410", e);
		}
		logger.info( LOG_FLAG + "HttpService[id:" + serviceDef.getId() + "] is started." + LOG_FLAG);
	}

	@Override
	public void close() {
		if (server != null)
			try {
				server.stop();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		// server.destroy();

	}

	/**
	 * 判断是否生成过class，如果没生成自动生成
	 */
	private void generateBeans() {
		// String beanProps[] =
		// getRequiredPropertyValue(CLASS_SYMBOL).split(",");
		// if (!StringUtil.isEmpty(beanProp) &&
		// !generatedMap.containsKey(beanProp))
		// if (!StringUtil.isEmpty(beanProp) )
		// {
		// / generatedMap.put(beanProp, true);
		logger.info( "Creating HttpServervlet classes.");

		DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<JavaFileObject>();
		logger.info( "Creating HttpServervlet classes1.");
		DynamicCompiler dynamicCompilerUtil = new DynamicCompiler();
		boolean compilerResult;
		try {
			/*
			 * String s =
			 * URLDecoder.decode(Thread.currentThread().getContextClassLoader
			 * ().getResource("").toURI().getPath() + "classes", "utf-8"); File
			 * f = new File(s); if (!f.exists()) f.mkdirs(); compilerResult =
			 * dynamicCompilerUtil.compiler("UTF-8",
			 * System.getProperty("java.class.path"), s, "",
			 * ClassPathUpdater.addClasspath(""), diagnostics);
			 */
			String s = URLDecoder.decode(Thread.currentThread().getContextClassLoader().getResource("").toURI().getPath() + "classes", "utf-8");
			File f = new File(s);
			if (!f.exists())
				f.mkdirs();
			String dirs = ClassPathUpdater.addClasspath("classes");
			File dirfile = new File(dirs);
			if (!dirfile.exists())
				dirfile.mkdirs();
			compilerResult = dynamicCompilerUtil.compiler("UTF-8", System.getProperty("java.class.path"), s, "", dirs, diagnostics);
			logger.info( "----编译开始-----");
			if (compilerResult) {
				logger.info( "----编译成功-----");
			} else {
				logger.info( "+++++编译失败++++++");
			}
		} catch (Exception e) {
			logger.info( "Creating HttpServervlet classes error.");
			e.printStackTrace();
			// throw new InterfaceException("01411", e);
		}
	}
}
