package com.brilliance.eibs.server;


import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.el.CommonUtil;
import com.brilliance.eibs.main.Client;
import com.sun.net.httpserver.*;
import org.apache.commons.io.IOUtils;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLParameters;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.security.*;
import java.security.cert.CertificateException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class HttpsService extends AbsServer {
    public static final String Key_Store_Type = "key_store_type";
    public static final String Def_Key_Store_Type = "JKS";
    public static final String Key_Manager_Factory = "key_manager_factory";
    public static final String Def_Key_Manager_Factory = "SunX509";
    public static final String Def_SSL_Context = "SSL";
    public static final String INTERFACENAME = "interfaceName";
    public static final String TRANSACTIONNAME = "transactionName";
    public static final String STOREPATH = "storepath";
    public static final String STOREPWD = "storepwd";
    public static final String ENCODE = "encode";
    public static final String DEF_ENCODE = "UTF-8";
    public static final String ROOT_PATH = "/";
    public static final int Def_BACKLOG = 1024;
    public static final String DELIMITER = ",";

    public HttpsService(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        super(context, serviceDef, elParser);
    }

    @Override
    public void run() {
        logger.info( LOG_FLAG + "starting HttpService[id:" + serviceDef.getId() + "]." + LOG_FLAG);
        try {
            String ksttyp = getPropertyValue(Key_Store_Type, Def_Key_Store_Type);
            String kmftyp = getPropertyValue(Key_Manager_Factory, Def_Key_Manager_Factory);
            final String encode = getPropertyValue(ENCODE, DEF_ENCODE);
            int backlog = getPropertyValue(REQUEST_QUEUE_LEN_SYMBOL, Def_BACKLOG);
            int port = Integer.parseInt(getRequiredPropertyValue(PORT_SYMBOL));
            String uri = getRequiredPropertyValue(URI_SYMBOL);
            String interfaceName = getRequiredPropertyValue(INTERFACENAME);
            String transactionName = getRequiredPropertyValue(TRANSACTIONNAME);
            final Map<String, String[]> handleMap = checkConfigValidity(uri, interfaceName, transactionName);
            String storepath = getRequiredPropertyValue(STOREPATH);
            String storepwd = getRequiredPropertyValue(STOREPWD);
            SSLContext sslContext = getSslContext(storepath, storepwd, ksttyp, kmftyp);

            HttpsServer server = HttpsServer.create(new InetSocketAddress(port), backlog);
            server.setHttpsConfigurator(new HttpsConfigurator(sslContext) {
                @Override
                public void configure(HttpsParameters params) {
                    SSLContext c = getSSLContext();
                    SSLParameters sslparams = c.getDefaultSSLParameters();
                    params.setSSLParameters(sslparams);
                }
            });
            server.setExecutor(new ThreadPoolExecutor(8, 200, 60, TimeUnit.SECONDS,
                    new ArrayBlockingQueue<Runnable>(backlog)));
            server.createContext(ROOT_PATH, new HttpHandler() {
                @Override
                public void handle(HttpExchange httpExchange) {
                    String dsturi = httpExchange.getRequestURI().getPath();
                    Headers header=httpExchange.getRequestHeaders();
                    try {
                        String requestBody = IOUtils.toString(httpExchange.getRequestBody(), encode);
                        for (Map.Entry<String, String[]> entry : handleMap.entrySet()) {
                            String orguri = entry.getKey();
                            if (dsturi.equals(orguri)) {
                                Object rtnVal = new Client().call(entry.getValue()[0], entry.getValue()[1], new Object[]{requestBody,header}).getContent();
                                if (rtnVal instanceof String) {
                                    response(httpExchange, (String) rtnVal, encode);
                                } else {
                                    response(httpExchange, CommonUtil.toJson(rtnVal), encode);
                                }
                                return;
                            }
                        }
                        logger.warn( "当前请求没有匹配的交易处理:" + dsturi);
                    } catch (Exception e) {
                        logger.error( "当前请求处理异常:" + dsturi, e);
                    }
                }
            });
            server.start();
        } catch (Exception e) {
            logger.error( "https server start failed:" + e.getMessage(), e);
            throw new InterfaceException("01410", "https server start failed", e);
        }
        logger.info( LOG_FLAG + "HttpsService[id:" + serviceDef.getId() + "] is started." + LOG_FLAG);
    }

    private Map<String, String[]> checkConfigValidity(String uri, String interfaceName, String transactionName) {
        String[] uris = uri.split(DELIMITER);
        String[] interfaceNames = interfaceName.split(DELIMITER);
        String[] transactionNames = transactionName.split(DELIMITER);
        if (uris.length == 0) {
            throw new InterfaceException("01410", "parameter [uri,interfaceName,transactionName] cannot be empty");
        }
        if (uris.length != interfaceNames.length || uris.length != transactionNames.length) {
            throw new InterfaceException("01410", "The number of three parameters does not match:[uri,interfaceName,transactionName]");
        }
        Map<String, String[]> rs = new HashMap<String, String[]>();
        for (int i = 0; i < uris.length; i++) {
            rs.put(uris[i], new String[]{interfaceNames[i], transactionNames[i]});
        }
        return rs;
    }

    @Override
    public void close() {

    }

    private SSLContext getSslContext(String storePath, String storePwd, String ksttyp, String kmftyp) throws KeyStoreException, IOException, NoSuchAlgorithmException, CertificateException, UnrecoverableKeyException, KeyManagementException {
        KeyStore ks = KeyStore.getInstance(ksttyp);
        //载入证书
        ks.load(new FileInputStream(storePath), storePwd.toCharArray());
        //建立一个密钥管理工厂
        KeyManagerFactory kmf = KeyManagerFactory.getInstance(kmftyp);
        //初始化工厂
        kmf.init(ks, storePwd.toCharArray());
        //建立证书实体
        SSLContext sslContext = SSLContext.getInstance(Def_SSL_Context);
        //初始化证书
        sslContext.init(kmf.getKeyManagers(), null, null);
        return sslContext;
    }

    private void response(HttpExchange xchg, String rtnmsg, String encode) {
        Headers resheader = xchg.getResponseHeaders();
        resheader.set("Content-type", "text/plain;charset=utf-8");
        resheader.set("Connection", "Keep-Alive");
        OutputStream os = null;
        try {
            int length = rtnmsg.getBytes(encode).length;
            xchg.sendResponseHeaders(200, length);
            os = xchg.getResponseBody();
            os.write(rtnmsg.getBytes(encode));
            os.flush();
        } catch (Exception e) {
            logger.error( "response occure exception:" + e.getMessage(), e);
            throw new InterfaceException("01410", "response occure exception", e);
        } finally {
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {
                    logger.error( "OutputStream close occure exception:" + e.getMessage(), e);
                    throw new InterfaceException("01410", "OutputStream close occure exception", e);
                }
            }
        }
    }
}
