package com.brilliance.eibs.server;

import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.server.link_v2_0.*;
import com.brilliance.eibs.util.LogUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class LinkServer extends AbsServer {

    private List<LinkDispatch> dispatchs = new ArrayList<LinkDispatch>();
    private ExecutorService executorService;

    public LinkServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        super(context, serviceDef, elParser);
    }

    @Override
    public void run() {
        logger.info(LOG_FLAG + "LinkServer starting ." + LOG_FLAG);
        String links = getRequiredPropertyValue(LINKS_SYMBOL);// 127.0.0.1:10001:10002
        int headTagLen = Integer.valueOf(getRequiredPropertyValue(HEAD_TAG_LEN_SYMBOL));
        String heartbeat = getPropertyValue(HEARTBEAT_SYMBOL);
        String fin = getPropertyValue(FIN_SYMBOL);
        long interval = Long.valueOf(getPropertyValue(INTERVAL_SYMBOL));
        long detect = Long.valueOf(getPropertyValue(DETECT_SYMBOL));
        int timeout = Integer.valueOf(getPropertyValue(TIMEOUT_SYMBOL));
        String interfaceName = getRequiredPropertyValue("interfaceName");
        String transcationId = getRequiredPropertyValue("transactionName");
        List<LinkData> datas = LinkParser.createSimplexLinkDatas(links);
        executorService = Executors.newFixedThreadPool(datas.size());
        for (LinkData data : datas) {
            Link link = new SimplexLink(data, LogUtil.getLogger(context));
            link.setTimeout(timeout);
            Heartbeat hb = new Heartbeat(interval, heartbeat.getBytes(), link);
            ReceiveDetect receiveDetect = new ReceiveDetect(detect, link);
            link.setHeartbeat(hb);
            link.setReceiveDetect(receiveDetect);
            LinkDispatch dispatch = new LinkDispatch(link, interfaceName, transcationId);
            dispatch.setHeadTagLen(headTagLen);
            dispatch.setFin(fin.getBytes());
            dispatchs.add(dispatch);
            executorService.submit(dispatch);
        }
        logger.info(LOG_FLAG + "LinkServer is finished." + LOG_FLAG);
    }

    @Override
    public void close() {
        for (LinkDispatch dispatch : dispatchs)
            dispatch.interrupted();
        executorService.shutdownNow();
    }

}
