package com.brilliance.eibs.server;

import com.brilliance.eibs.core.frame.SocketProxyFactry;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.server.socket.IServerMode;
import com.brilliance.eibs.server.socket.impl.SocketServerMode;
import com.brilliance.eibs.util.LogUtil;

/**
 * socket服务
 *
 * @author gechengyang
 */
public class ShortSocketServer extends AbsServer {

    private int port;
    private int queueLen;
    private int size;
    private String delayTime;
    private String interfaceName;
    private String transName;
    private int timeout;
    private IServerMode mode;

    private boolean has_head = true;// 是否有报文头
    private String head_len_type = IServerInstance.HEAD_LEN_TYPE_10;// 报文头长度是二进制还是十进制
    private int head_len = 0;// 报文头长度
    private boolean is_contain_head_len = false;// 报文头长度是否包含报文头
    private int fill_len = 0;// 默认为0
    private boolean is_contain_fill_len = false;// 报文头长度是否包含报文头和报文体之间数据的长度 true
    // false
    private int body_offset = 0;// 报文体位移量 默认为0, -1为减去1个长度，+1为增加1个长度

    private String encoding = "UTF-8";

    public ShortSocketServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        super(context, serviceDef, elParser);
    }

    @Override
    public void run() {
        logger.info(LOG_FLAG + "SocketServer starting ." + LOG_FLAG);
        // 服务发布的端口
        port = Integer.valueOf(getRequiredPropertyValue(PORT_SYMBOL));
        logger.info("socketserver port=" + port);
        // 等待队列的长度
        queueLen = Integer.valueOf(getRequiredPropertyValue(REQUEST_QUEUE_LEN_SYMBOL));
        size = Integer.valueOf(getRequiredPropertyValue(THREAD_POOL_SIZE_SYMBOL));

        String trustips = getPropertyValue(TRUST_IP_SYMBOL);
        // 延迟启动时间
        delayTime = getPropertyValue(DELAY_TIME);
        String type = getPropertyValue(TYPE);
        interfaceName = getRequiredPropertyValue("interfaceName");
        transName = getRequiredPropertyValue("transactionName");
        timeout = Integer.valueOf(getRequiredPropertyValue(RECEIVE_TIMEOUT_SYMBOL));

        has_head = getPropertyValue(HAS_CONTAIN_HEAD, true);
        head_len_type = getPropertyValue(HEAD_LEN_TYPE, HEAD_LEN_TYPE_10);
        head_len = getPropertyValue(HEAD_LEN, 0);
        is_contain_head_len = getPropertyValue(IS_CONTAIN_HEAD_LEN, false);
        fill_len = getPropertyValue(FILL_LEN, 0);
        is_contain_fill_len = getPropertyValue(IS_CONTAIN_FILL_LEN, false);
        body_offset = getPropertyValue(BODY_OFFSET, 0);
        encoding = getPropertyValue(ENCODING, "UTF-8");

        if (SocketProxyFactry.MINA_TYPE.equals(type)) {

        } else if (SocketProxyFactry.ORG_SOCKET_TYPE.equals(type)) {
            mode = new SocketServerMode(SocketProxyFactry.SHORT_SOCKET_TYPE, trustips, port, queueLen, size, delayTime, interfaceName, transName,
                    timeout, serviceDef.getExpression(), args, serviceDef.getId(), LogUtil.getLogger(context), has_head, head_len_type,
                    head_len, is_contain_head_len, fill_len, is_contain_fill_len, body_offset, encoding);
        }
        mode.startup();

        logger.info(LOG_FLAG + "SocketServer is finished." + LOG_FLAG);
    }

    /**
     * 关闭ServerSocket
     */
    @Override
    public void close() {
        mode.close();
    }

}
