package com.brilliance.eibs.server;

import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.server.connection.Detect;
import com.brilliance.eibs.server.connection.FilterChain;
import com.brilliance.eibs.server.connection.impl.TransFilter;
import com.brilliance.eibs.server.terminal.Terminal;
import com.brilliance.eibs.server.terminal.TerminalPool;
import com.brilliance.eibs.server.terminal.impl.AsyncFixedSocketTerminal;

import java.io.IOException;

public class TerminalServer extends AbsServer {

    private Terminal terminal;
    private TerminalListner listener;

    public TerminalServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        super(context, serviceDef, elParser);
    }

    @Override
    public void run() {
        logger.info(LOG_FLAG + "TerminalServer starting ." + LOG_FLAG);
        int port = Integer.valueOf(getRequiredPropertyValue("port"));
        String ip = getRequiredPropertyValue("ip");
        String interfaceName = getRequiredPropertyValue("interfaceName");
        String transactionId = getRequiredPropertyValue("transactionName");
        int length = Integer.valueOf(getRequiredPropertyValue("length"));
        FilterChain chain = new FilterChain();
        chain.addResponseFilter(new TransFilter(interfaceName, transactionId));
        terminal = new AsyncFixedSocketTerminal(ip, port, length, chain);
        TerminalPool.addTerminal(serviceDef.getId(), terminal);

        listener = new TerminalListner();
        Thread t = new Thread(listener);
        t.setName("Asynchronous Terminal '" + serviceDef.getId() + "' Server Listener");
        t.setDaemon(true);
        t.start();

        terminal.startup();
        logger.info(LOG_FLAG + "TerminalServer is finished." + LOG_FLAG);
    }

    @Override
    public void close() {
        listener.interrupted();
        terminal.close();
        TerminalPool.removeTerminal(terminal);
    }

    private class TerminalListner extends Detect {
        private long interval = 60000;

        private TerminalListner() {
        }

        private TerminalListner(long interval) {
            this.interval = interval;
        }

        @Override
        protected void exception() {
            interrupted();
        }

        @Override
        protected void handle() throws IOException {
            try {
                Thread.sleep(interval);
                logger.info("[" + serviceDef.getId() + "] terminal status : " + terminal.isConnected());
            } catch (InterruptedException e) {
                logger.error(e.getMessage(), e);
            }
        }

        @Override
        protected void close() {
            logger.info("TerminalListner of [" + serviceDef.getId() + "] terminal is closed.");
        }

    }

}
